//
//  FaceMovementTrackingVC.swift
//  ARKitDemo
//
//  Created by Jay on 28/09/19.
//  Copyright © 2019 Jay. All rights reserved.
//

import UIKit
import ARKit
import SceneKit

@available(iOS 11.0, *)
class FaceMovementTrackingVC: UIViewController {
    
    @IBOutlet weak var btnOK: UIButton!
    @IBOutlet weak var btnEyeBlink: UIButton!
    
    @IBOutlet var sceneView: ARSCNView!
    @IBOutlet weak var viewInfo: UIView!
    
    var isEnable : Bool = false
    let features = [" ", "nose", "leftEye", "rightEye", "mouth", "hat"]
    let featureIndices = [[9], [1064], [42], [24, 25], [20]]
    
    override func viewDidLoad() {
        super.viewDidLoad()
        guard ARFaceTrackingConfiguration.isSupported else { fatalError() }
        sceneView.delegate = self
        sceneView.isPlaying = true
//        sceneView.loops = true
        
        btnOK.layer.cornerRadius = 5.0
        btnOK.layer.borderWidth = 2.0
        btnOK.layer.borderColor = #colorLiteral(red: 1.0, green: 1.0, blue: 1.0, alpha: 1.0)
        
        btnEyeBlink.layer.cornerRadius = 5.0
        btnEyeBlink.layer.borderWidth = 2.0
        btnEyeBlink.layer.borderColor = #colorLiteral(red: 1.0, green: 1.0, blue: 1.0, alpha: 1.0)
        // Do any additional setup after loading the view.
    }
    override func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        let configuration = ARFaceTrackingConfiguration()
        sceneView.session.run(configuration)
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        super.viewWillDisappear(animated)
        sceneView.session.pause()
    }
    @IBAction func btnOKPressed(_ sender: Any) {
        viewInfo.isHidden = true
        self.isEnable = true
    }
    @IBAction func btnEyeBlinkViewPressed(_ sender: Any) {
    }
    
    func updateFeatures(for node: SCNNode, using anchor: ARFaceAnchor) {
        for (feature, indices) in zip(features, featureIndices) {
            let child = node.childNode(withName: feature, recursively: false) as? EmojiNode
            let vertices = indices.map { anchor.geometry.vertices[$0] }
            child?.updatePosition(for: vertices)
            
            switch feature {
            case "leftEye":
                let scaleX = child?.scale.x ?? 1.0
                let eyeBlinkValue = anchor.blendShapes[.eyeBlinkLeft]?.floatValue ?? 0.0
                child?.scale = SCNVector3(scaleX, 1.0 - eyeBlinkValue, 1.0)
                // print("left Eye Value \(eyeBlinkValue)")
            // print("left Eye scale \(child?.scale)")
            case "rightEye":
                let scaleX = child?.scale.x ?? 1.0
                let eyeBlinkValue = anchor.blendShapes[.eyeBlinkRight]?.floatValue ?? 0.0
                child?.scale = SCNVector3(scaleX, 1.0 - eyeBlinkValue, 1.0)
                // print("Right Eye Value \(eyeBlinkValue)")
                // print("left Eye scale \(child?.scale)")
                
            case "mouth":
                let jawOpenValue = anchor.blendShapes[.jawOpen]?.floatValue ?? 0.2
                child?.scale = SCNVector3(1.0, 0.8 + jawOpenValue, 1.0)
            default:
                break
            }
        }
    }
    
}

@available(iOS 11.0, *)
extension FaceMovementTrackingVC: ARSCNViewDelegate {
    
    func renderer(_ renderer: SCNSceneRenderer, nodeFor anchor: ARAnchor) -> SCNNode? {
        guard let faceAnchor = anchor as? ARFaceAnchor,
            let device = sceneView.device else { return nil }
        let faceGeometry = ARSCNFaceGeometry(device: device)
        let node = SCNNode(geometry: faceGeometry)
        node.geometry?.firstMaterial?.fillMode = .lines
        node.geometry?.firstMaterial?.transparency = 0.0
        //        let noseNode = EmojiNode(with: noseOptions)
        //        noseNode.name = "nose"
        //        node.addChildNode(noseNode)
        //
        //        let leftEyeNode = EmojiNode(with: eyeOptions)
        //        leftEyeNode.name = "leftEye"
        //        leftEyeNode.rotation = SCNVector4(0, 1, 0, GLKMathDegreesToRadians(180.0))
        //        node.addChildNode(leftEyeNode)
        //
        //        let rightEyeNode = EmojiNode(with: eyeOptions)
        //        rightEyeNode.name = "rightEye"
        //        node.addChildNode(rightEyeNode)
        //
        //        let mouthNode = EmojiNode(with: mouthOptions)
        //        mouthNode.name = "mouth"
        //        node.addChildNode(mouthNode)
        //
        //        let hatNode = EmojiNode(with: hatOptions)
        //        hatNode.name = "hat"
        //        node.addChildNode(hatNode)
        
        updateFeatures(for: node, using: faceAnchor)
        return node
    }
    
    func renderer(_ renderer: SCNSceneRenderer, didUpdate node: SCNNode, for anchor: ARAnchor) {
        guard let faceAnchor = anchor as? ARFaceAnchor, let faceGeometry = node.geometry as? ARSCNFaceGeometry else { return }
        
        faceGeometry.update(from: faceAnchor.geometry)
        //if self.initialEyeMode != .firstTimeOnly{
        expression(anchor: faceAnchor)
        //}
        if let faceAnchor = anchor as? ARFaceAnchor {
            DispatchQueue.main.async {
                //This changes as I move camera and cannot able to detect logic
                let position = faceAnchor.transform.position()
                print ("X: \(node.position.x)----Y: \(node.position.y)")
                print ("X: \(position.x)----Y: \(position.y)")
            }
            
            if node.position.x > -0.8 {
                let alert = UIAlertController(title: "Face Movement", message: "Face moved to left", preferredStyle: UIAlertController.Style.alert)
                alert.addAction(UIAlertAction(title: "OK", style: UIAlertAction.Style.default, handler: nil))
                self.present(alert, animated: true, completion: nil)
            }
        }
        updateFeatures(for: node, using: faceAnchor)
    }
    
    
    func expression(anchor: ARFaceAnchor) {
        // let smileLeft = anchor.blendShapes[.mouthSmileLeft]
        //  let smileRight = anchor.blendShapes[.mouthSmileRight]
        let leftEyeBlink = anchor.blendShapes[.eyeBlinkLeft]
        let rightEyeBlink = anchor.blendShapes[.eyeBlinkRight]
        //        let cheekPuff = anchor.blendShapes[.cheekPuff]
        //        let tongue = anchor.blendShapes[.tongueOut]
        //        self.analysis = ""
        
        
        //   if  (((leftEyeBlink?.decimalValue ?? 0.0) + (rightEyeBlink?.decimalValue ?? 0.0)) > 0.9) {
        // self.analysis += "Your eyes blink. "
        // create the alert
        // print("Eyes Close")
        
        //            if self.initialEyeMode == nil {
        //                self.initialEyeMode = .closed
        //            } else {
        //                if self.initialEyeMode == .opened && self.isEnable {
        //                    DispatchQueue.main.async {
        //                        if self.initialEyeMode != .firstTimeOnly {
        //                            DispatchQueue.main.asyncAfter(deadline: .now() + 1) {
        //                                let image = self.sceneView.snapshot()
        //                                UIImageWriteToSavedPhotosAlbum(image, self, #selector(self.image(_:didFinishSavingWithError:contextInfo:)), nil)
        //                            }
        //                        }
        //                        self.initialEyeMode = .firstTimeOnly
        //                    }
        //                }
        //            }
        //        }else{
        //            //print("Eyes Open")
        //            if self.initialEyeMode == nil {
        //                self.initialEyeMode = .opened
        //            } else {
        //                if self.initialEyeMode == .closed && self.isEnable {
        //                    DispatchQueue.main.async {
        //                        if self.initialEyeMode != .firstTimeOnly {
        //                            DispatchQueue.main.asyncAfter(deadline: .now() + 1) {
        //                                let image = self.sceneView.snapshot()
        //                                UIImageWriteToSavedPhotosAlbum(image, self, #selector(self.image(_:didFinishSavingWithError:contextInfo:)), nil)
        //                            }
        //                        }
        //                        self.initialEyeMode = .firstTimeOnly
        //                    }
        //                }
        //            }
        //        }
        
        //        if ((smileLeft?.decimalValue ?? 0.0) + (smileRight?.decimalValue ?? 0.0)) > 0.9 {
        //            self.analysis += "You are smiling. "
        //        }
        //
        //        if cheekPuff?.decimalValue ?? 0.0 > 0.1 {
        //            self.analysis += "Your cheeks are puffed. "
        //        }
        //
        //        if tongue?.decimalValue ?? 0.0 > 0.1 {
        //            self.analysis += "Don't stick your tongue out! "
        //        }
    }
}

extension matrix_float4x4 {
    func position() -> SCNVector3 {
        return SCNVector3(columns.3.x, columns.3.y, columns.3.z)
    }
}
